/***************************************************************************
 *                                                                         *
 *  MAXIMUS-CBCS Source Code, Version 1.02                                 *
 *  Copyright 1989, 1990 by Scott J. Dudley.  All rights reserved.         *
 *                                                                         *
 *  Conversion program for Opus-style SYSTEM*.BBS/DAT files                *
 *                                                                         *
 *  For complete details of the licensing restrictions, please refer to    *
 *  the licence agreement, which is published in its entirety in MAX.C     *
 *  and LICENCE.MAX.                                                       *
 *                                                                         *
 *  USE OF THIS FILE IS SUBJECT TO THE RESTRICTIONS CONTAINED IN THE       *
 *  MAXIMUS-CBCS LICENSING AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF THIS *
 *  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES, OR IF YOU DO NOT HAVE   *
 *  THESE FILES, YOU SHOULD IMMEDIATELY CONTACT THE AUTHOR AT ONE OF THE   *
 *  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO USE THIS    *
 *  FILE WITHOUT HAVING ACCEPTED THE TERMS OF THE MAXIMUS-CBCS LICENSING   *
 *  AGREEMENT, OR SUCH OTHER AGREEMENT AS YOU ARE ABLE TO REACH WITH THE   *
 *  AUTHOR.                                                                *
 *                                                                         *
 *  You can contact the author at one of the address listed below:         *
 *                                                                         *
 *  Scott Dudley           FidoNet  1:249/106                              *
 *  777 Downing St.        IMEXnet  89:483/202                             *
 *  Kingston, Ont.         Internet f106.n249.z1.fidonet.org               *
 *  Canada - K7M 5N3       BBS      (613) 389-8315 - HST/14.4K, 24hrs      *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "prog.h"
#include "ffind.h"
#include "max.h"
#include "cvtusr.h"

struct _ab
{
  char path[60];
  char tag[40];
};

#define NUM_ABBS 512

/* for qsort */
static int stcomp(char *arg1,char *arg2)
{
  static char a1[10],
              a2[10];

  if (strlen(arg1) <= 1)
  {
    a1[0]='0';
    a1[1]=*arg1;
    a1[2]='\0';
  }
  else strcpy(a1,arg1);

  if (strlen(arg2) <= 1)
  {
    a2[0]='0';
    a2[1]=*arg2;
    a2[2]='\0';
  }
  else strcpy(a2,arg2);

  return (strcmp(a1,a2));
}


int OpusToMaxPriv(byte opriv);


int main(int argc,char *argv[])
{
  FILE *areasfile,
       *dirbbs,
       *areasbbs;

  FFIND *ff,
        *ffo;

  struct _sys sys;
  struct _sys110 sys110;
  static struct _ab abbs[NUM_ABBS];

  char temp[80],
       *p;

  int x,
      ret,
      sysfile,
      maxsys,
      abbs_num,
      dat,
      sn;

  char sysname[255][3];

  (void)argc;
  (void)argv;

  abbs_num=0;

  Hello("PIPER","A Maximus-CBCS Control File Generator",VERSION,"1990");

  if (fexist("AREAS.CTL"))
  {
    printf("`AREAS.CTL' already exists;  Overwrite [y,N]? ");
    fgets(temp,80,stdin);

    printf("\n");

    if (toupper(*temp) != 'Y')
    {
      printf("Aborted!\n");
      return -1;
    }

    if (fexist("AREAS.BAK"))
      unlink("AREAS.BAK");

    rename("AREAS.CTL","AREAS.BAK");
  }

  if ((areasbbs=fopen("AREAS.BBS","r")) != NULL ||
      (areasbbs=fopen("ECHO.CTL","r")) != NULL)
  {
    while (fgets(temp,80,areasbbs) != NULL)
    {
      if (abbs_num >= NUM_ABBS-1)
      {
        printf("Too many areas in echomail control file -- continuing...\n");
        break;
      }

      if (strchr(temp,';'))
        *strchr(temp,';')='\0';

      if (! isnl(temp))
      {
        p=temp;

        /* Passthrough areas... */
        while (*p=='#' || isdigit(*p) || isspace(*p))
          p++;

        getword(strupr(p),abbs[abbs_num].path," \t\n",1);

        if (abbs[abbs_num].path[strlen(abbs[abbs_num].path)-1] != '\\')
          strcat(abbs[abbs_num].path,"\\");;

        getword(p,abbs[abbs_num++].tag," \t\n",2);
      }
    }

    fclose(areasbbs);
  }

  if (!fexist("SYSTEM*.BBS") && !fexist("SYSTEM*.DAT"))
  {
    printf("No SYSTEMxx.BBS/DAT files exist.  Continue anyway [y,N]? ");
    fgets(temp,80,stdin);

    printf("\n");

    if (toupper(*temp) != 'Y')
    {
      printf("Aborted!\n");
      return -1;
    }
  }

  if ((areasfile=fopen("AREAS.CTL","w"))==NULL)
  {
    printf("Fatal error opening `AREAS.CTL' for write!\n");
    exit(1);
  }

  printf("Scanning system files...\n");

  ff=FindOpen("SYSTEM*.BBS",0);

  if (ff)
    dat=FALSE;
  else
  {
    ff=FindOpen("SYSTEM*.DAT",0);
    dat=TRUE;
  }

  maxsys=0;

  if (ff)
  {
    for (ret=0;ret==0;maxsys++)
    {
      if (ff->ff_name[6]=='.')   /* SYSTEM.BBS */
      {
        sysname[maxsys][0]='0';
        sysname[maxsys][1]='\0';
      }
      else
      {
        sysname[maxsys][0]=ff->ff_name[6];

        if (ff->ff_name[7]=='.')
          sysname[maxsys][1]='\0';
        else
        {
          sysname[maxsys][1]=ff->ff_name[7];
          sysname[maxsys][2]='\0';
        }
      }

      ret=FindNext(ff);
    }

    FindClose(ff);
  }

  qsort(sysname,maxsys,sizeof(sysname[0]),stcomp);

  for (sn=0;sn < maxsys;sn++)
  {
    printf("\rProcessing area %s...  ",sysname[sn]);

    if (dat)
    {
      sscanf(sysname[sn],"%x",&x);

      fprintf(areasfile,"Area %d\n",x);

      sprintf(temp,"SYSTEM%s.DAT",sysname[sn]);

      if ((sysfile=open(temp,O_RDONLY | O_BINARY))==-1)
        printf("Can't open `%s' -- Continuing...\n",temp);
      else
      {
        read(sysfile,(char *)&sys110,sizeof(struct _sys110));
        close(sysfile);

        if (*sys110.msgpath)
          fprintf(areasfile,"        MsgAccess       %s\n",Priv_Level(OpusToMaxPriv(sys110.MsgPriv)));

        if (*sys110.filepath)
        {
          fprintf(areasfile,"        FileAccess      %s\n\n",Priv_Level(OpusToMaxPriv(sys110.FilePriv)));
          fprintf(areasfile,"        FileInfo        %s\n",sys110.filtitle);
          fprintf(areasfile,"        Download        %s\n",sys110.filepath);
        }

        if (*sys110.uppath)
          fprintf(areasfile,"        Upload          %s\n",sys110.uppath);

        if (*sys110.listpath)
          fprintf(areasfile,"        FileList        %s\n",sys110.listpath);

        if (*sys110.msgpath)
        {
          fprintf(areasfile,"\n        MsgInfo         %s\n",sys110.msgtitle);

          sprintf(temp,"%sORIGIN.*",sys110.msgpath);

          ffo=FindOpen(temp,0);

          if (ffo && (sys110.attrib & ECHOMAIL))
          {
            sprintf(temp,"%s%s",sys110.msgpath,ffo->ff_name);

            if ((dirbbs=fopen(temp,"r")) != NULL)
            {
              fgets(temp,80,dirbbs);
              fclose(dirbbs);

              if (temp[x=strlen(temp)-1]=='\n')
                temp[x]='\0';

              if (strlen(ffo->ff_name) <= 7)
                x=0;
              else if (sscanf(ffo->ff_name,"ORIGIN.%x",&x) != 1)
                x=0;

              fprintf(areasfile,"        Origin          %d %s\n",x,temp);
            }
          }

          FindClose(ffo);

          if ((sys110.attrib & ECHOMAIL) && *sys110.EchoName)
            fprintf(areasfile,"        MsgName         %s\n",sys110.EchoName);

          if (*sys110.barrpath)
            fprintf(areasfile,"        Barricade       %s\n",sys110.barrpath);

          if (sys110.attrib & ECHOMAIL)
            strcpy(temp,   "EchoMail");
          else if (sys110.attrib & SYSMAIL)
            strcpy(temp,   "Matrix  ");
          else strcpy(temp,"Local   ");

          fprintf(areasfile,  "        %s        %s\n\n",temp,sys110.msgpath);

          if ((sys110.attrib & NOPRIVATE) && (sys110.attrib & NOPUBLIC))
            fprintf(areasfile,"        Read-Only\n");
          else if (sys110.attrib & NOPRIVATE)
            fprintf(areasfile,"        Public Only\n");
          else if (sys110.attrib & NOPUBLIC)
            fprintf(areasfile,"        Private Only\n");
          else fprintf(areasfile,"        Public and Private\n");

          if (sys110.attrib & ANON_OK)
            fprintf(areasfile,"        Anonymous OK\n");
        }
      }
    }
    else
    {
      fprintf(areasfile,"Area %s\n",sysname[sn]);

      if (eqstri(sysname[sn],"0"))
        strcpy(temp,"SYSTEM.BBS");
      else sprintf(temp,"SYSTEM%s.BBS",sysname[sn]);

      if ((sysfile=open(temp,O_RDONLY | O_BINARY))==-1)
        printf("Can't open `%s' -- Continuing...\n",temp);
      else
      {
        read(sysfile,(char *)&sys,sizeof(struct _sys));
        close(sysfile);

        if (*sys.msgpath)
          fprintf(areasfile,"        MsgAccess       %s\n",Priv_Level(sys.priv));

        if (*sys.filepath)
          fprintf(areasfile,"        FileAccess      %s\n",Priv_Level(sys.priv));

        if (*sys.filepath)
        {
          fprintf(areasfile,"\n");

          sprintf(temp,"%sDIR.BBS",sys.filepath);

          if ((dirbbs=fopen(temp,"r")) != NULL)
          {
            fgets(temp,80,dirbbs);
            fclose(dirbbs);

            if (temp[x=strlen(temp)-1]=='\n')
              temp[x]='\0';

            fprintf(areasfile,"        FileInfo        %s\n",temp);
          }

          fprintf(areasfile,  "        Download        %s\n",sys.filepath);
        }

        if (*sys.uppath)
          fprintf(areasfile,  "        Upload          %s\n",sys.uppath);

        if (*sys.msgpath)
        {
          sprintf(temp,"%sDIR.BBS",sys.msgpath);

          fprintf(areasfile,"\n");

          if ((dirbbs=fopen(temp,"r")) != NULL)
          {
            fgets(temp,80,dirbbs);
            fclose(dirbbs);

            if (temp[x=strlen(temp)-1]=='\n')
              temp[x]='\0';

            fprintf(areasfile,"        MsgInfo         %s\n",temp);
          }

          sprintf(temp,"%sORIGIN.*",sys.msgpath);

          ffo=FindOpen(temp,0);

          if (ffo && (sys.attrib & ECHOMAIL))
          {
            sprintf(temp,"%s%s",sys.msgpath,ffo->ff_name);

            if ((dirbbs=fopen(temp,"r")) != NULL)
            {
              fgets(temp,80,dirbbs);
              fclose(dirbbs);

              if (temp[x=strlen(temp)-1]=='\n')
                temp[x]='\0';

              if (strlen(ffo->ff_name) <= 7)
                x=0;
              else if (sscanf(ffo->ff_name,"ORIGIN.%x",&x) != 1)
                x=0;

              fprintf(areasfile,"        Origin          %d %s\n",x,temp);
            }
          }

          FindClose(ffo);

          if (sys.attrib & ECHOMAIL)
          {
            for (x=0;x < abbs_num;x++)
              if (eqstri(sys.msgpath,abbs[x].path))
              {
                fprintf(areasfile,"        MsgName         %s\n",abbs[x].tag);
                break;
              }

            if (x==abbs_num)
              fprintf(areasfile,    "        MsgName         ?\n");
          }

          /* If the area is currently barricaded, or the BBSPATH points to   *
           * a file, instead of a path.                                      */

          if (fexist(sys.bbspath))
            fprintf(areasfile,"        Barricade       %s\n",sys.bbspath);

          if (sys.attrib & ECHOMAIL)
            strcpy(temp,   "EchoMail");
          else if (sys.attrib & SYSMAIL)
            strcpy(temp,   "Matrix  ");
          else strcpy(temp,"Local   ");

          fprintf(areasfile,  "        %s        %s\n\n",temp,sys.msgpath);

          if ((sys.attrib & NOPRIVATE) && (sys.attrib & NOPUBLIC))
            fprintf(areasfile,"        Read-Only\n");
          else if (sys.attrib & NOPRIVATE)
            fprintf(areasfile,"        Public Only\n");
          else if (sys.attrib & NOPUBLIC)
            fprintf(areasfile,"        Private Only\n");
          else fprintf(areasfile,"        Public and Private\n");

          if (sys.attrib & ANON_OK)
            fprintf(areasfile,"        Anonymous OK\n");
        }
      }
    }

    fprintf(areasfile,"End Area\n\n");
  }

  fclose(areasfile);

  printf("\r                     \rAll done!\n");

  return 0;
}

int OpusToMaxPriv(byte opriv)
{
  switch(opriv)
  {
    case _SYSOP:
      return SYSOP;

    case _ASSTSYSOP:
      return ASSTSYSOP;

    case _CLERK:
      return CLERK;

    case _EXTRA:
      return EXTRA;

    case _FAVORED:
      return FAVORED;

    case _PRIVEL:
      return PRIVIL;

    case _WORTHY:
      return WORTHY;

    case _NORMAL:
      return NORMAL;

    case _LIMITED:
      return LIMITED;

    case _DISGRACE:
      return DISGRACE;

    case _TWIT:
      return TWIT;

    default:
      return HIDDEN;
  }
}



